@@ -23,9 +23,9 @@ class.  For those wishing to override the speech generators, however,
 one can create a new instance and replace/extend the speech generators
 as they see fit."""
 
-__id__        = "$Id: speechgenerator.py 3278 2007-11-29 19:07:04Z shaeger $"
-__version__   = "$Revision: 3278 $"
-__date__      = "$Date: 2007-11-29 14:07:04 -0500 (Thu, 29 Nov 2007) $"
+__id__        = "$Id: speechgenerator.py 3378 2007-12-14 20:20:12Z wwalker $"
+__version__   = "$Revision: 3378 $"
+__date__      = "$Date: 2007-12-14 15:20:12 -0500 (Fri, 14 Dec 2007) $"
 __copyright__ = "Copyright (c) 2005-2007 Sun Microsystems Inc."
 __license__   = "LGPL"
 
@@ -161,21 +161,12 @@ class SpeechGenerator:
         Returns a list of utterances to be spoken.
         """
 
-        result = self._script.getAcceleratorAndShortcut(obj)
-
-        accelerator = result[0]
-        #shortcut = result[1]
-
         utterances = []
 
-        # [[[TODO: WDW - various stuff preserved while we work out the
-        # desired verbosity here.]]]
-        #
-        #if len(shortcut) > 0:
-        #    utterances.append(_("shortcut") + " " + shortcut)
-        #    utterances.append(accelerator)
+        result = self._script.getKeyBinding(obj)
+        accelerator = result[2]
+
         if len(accelerator) > 0:
-            #utterances += (_("accelerator") + " " + accelerator)
             utterances.append(accelerator)
 
         return utterances
